﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;

public enum GameMode {None, Bear, Bird, GameOver};


public class BearLogic : MonoBehaviour {

	public static BearLogic S;
	public GameMode mode = GameMode.None;

	public GameObject bearPrefab;
	private Transform bearTransform;
	private Vector3 bearPosition;
	private GameObject bearInstance;
	private Animator bearInstanceAnim;

	public GameObject pooParticle;
	public float pooForce = 500.0f;


	// Cameras
	public Camera bearCamera;
	public Camera birdCamera;
	public GameObject bird;

	private Vector3 birdRise;
	private Vector3 birdFocus;

	private Vector3 infCenter;
	private Vector3 xVelocity = Vector3.zero;

	void Awake() {
		S = this; // define the singleton
	}

	// Use this for initialization
	void Start () {
		LevelReset ();

		/*
		// start as a bear
		mode = GameMode.Bear;

		// set bear position and camera
		bearPosition = Vector3.zero;
		bearCamera.enabled = true;
		birdCamera.enabled = false;


		birdRise = new Vector3 (0, .02f, -0.01f);
		*/
	}

	// Update is called once per frame
	void Update () {
		// adjust camera focus if needed 
		AdjustFocus();
		// move the camera as needed
		MoveCamera ();

		// bear debug
		// DrawBearHeading();

		/*
		 * 
		 * Removed at 11:49am due to poor implementation.  clean up later


		// check for restart
		if (Input.GetKeyDown (KeyCode.R)) {
			//kill the singleton
			Reset();
		}

*/

	}

	void LevelReset() {
		// start as a bear
		mode = GameMode.Bear;

		// set bear position and camera
		bearPosition = Vector3.zero;
		bearCamera.enabled = true;
		birdCamera.enabled = false;

		birdRise = new Vector3 (0, .02f, -0.01f);
	}

	void Reset() {
		LevelReset ();
		// reset the main player if he exists.
		if (bearInstance) {
			Destroy (bearInstance);
		}

		GameObject[] bearList = GameObject.FindGameObjectsWithTag ("Bear");
		foreach (GameObject bear in bearList) {
			bear.GetComponent<OtherBearControls>().Reset ();
		}

	}


	void MoveCamera() {
		if (mode == GameMode.Bird) {
			bird.transform.position += birdRise;
			bird.transform.LookAt (birdFocus);
		}
	}

	void AdjustFocus() {
		// run the script to reset where the bounds should be.
		birdFocus = Vector3.SmoothDamp(birdFocus, infCenter, ref xVelocity, 3.0f);
		// cameraposition.x = Mathf.SmoothDamp (cameraposition.x, playerposition.x, ref xVelocity, 0.5f);
	}

	public IEnumerator EvalCamera() {
		while (mode == GameMode.Bird) {

			// Yield for a few seconds
			yield return new WaitForSeconds(3.0f);

			// runs while in bird mode
			// make the new Bounds
			Bounds totalBounds = new Bounds(birdFocus, Vector3.zero); // BOUNDS needs a center point (the current focus) and a size (ZeroVector).  Blank instantiation includes 0,0,0 as a point, which is a bug

			GameObject[] infected = GameObject.FindGameObjectsWithTag ("Poo");

			// don't run until another is infected.
			if (infected.Length > 1) {  
				// for each "infected" item...
				foreach (GameObject animal in infected) {
					// get the transform & encapsulate
					totalBounds.Encapsulate (animal.transform.position);
				}
				// Find and set infCenter
				infCenter = totalBounds.center;

				Debug.Log ("infection center: " + infCenter);
			
			}


		}
	}

	public void BearPooped(GameObject bear) {
		// Accepts a "this" object for the bear
		// Defines the position of the bear
		bearTransform = bear.transform;

		bearPosition = bear.transform.position;
		Debug.Log("Bear Location: " + bearPosition);

		// Changes the camera to bird
		bearCamera.enabled = false;
		birdCamera.enabled = true;

		// Set the position of bird
		Vector3 birdStart = new Vector3 (0,5.0f,-15.0f);
		birdCamera.transform.position = bearPosition + birdStart;
		birdFocus = bearPosition;
		bird.transform.LookAt (birdFocus);
		infCenter = birdFocus;  // set the initial value
		// bird.LookAt (bearPosition);

		// Changes the game state to bird, starts the constant re-evaluation
		mode = GameMode.Bird;
		// start figuring out the infected
		StartCoroutine (EvalCamera ());



		// Poops

		bearInstance = Instantiate(bearPrefab, bearPosition, bear.transform.rotation);
		// Instantiate Poop Parts and explode

		bearInstanceAnim = bearInstance.GetComponent<Animator> ();
		bearInstanceAnim.SetBool ("isWalking", false);
		bearInstanceAnim.SetBool ("isDead", true);

		// set infection
		bearInstance.tag = "Infected";


		StartCoroutine(PooSplosion (bear.transform));

		// start the music
		SoundManager.S.playMusic();


	}

	/*
	void PooSplosion(Transform pooSource) {
		for (int i = 0; i < 10; i++) {
			Instantiate (pooParticle, pooSource.position, pooSource.rotation);
		}
	}



	public IEnumerator PooSplosion(Transform pooSource) {
		for (int i = 0; i < 15; i++) {
			
			// Set the spawn point
			Vector3 nugDirection = bearInstance.transform.forward;
			Vector3 nugSpawn = (pooSource.position - (3 * nugDirection) + bearInstance.transform.up);

			// Spawn the poo particle
			// GameObject nugget = Instantiate (pooParticle, nugSpawn, Quaternion.Inverse (pooSource.rotation));
			GameObject nugget = Instantiate (pooParticle, nugSpawn, pooSource.rotation);


			Rigidbody nuggetRB = nugget.GetComponent<Rigidbody> ();
			nuggetRB.AddForce ((nugDirection * -pooForce) + (Random.insideUnitSphere * pooForce * 0.4f));
			yield return new WaitForSeconds (0.05f);
		}

	}
*/
	public IEnumerator PooSplosion(Transform pooSource) {
		for (int i = 0; i < 15; i++) {

			// Set the spawn point
			Vector3 nugDirection = pooSource.forward;
			Vector3 nugSpawn = (pooSource.position - (3 * nugDirection) + pooSource.up);

			// Spawn the poo particle
			// GameObject nugget = Instantiate (pooParticle, nugSpawn, Quaternion.Inverse (pooSource.rotation));
			GameObject nugget = Instantiate (pooParticle, nugSpawn, pooSource.rotation);


			Rigidbody nuggetRB = nugget.GetComponent<Rigidbody> ();
			nuggetRB.AddForce ((nugDirection * -pooForce) + (Random.insideUnitSphere * pooForce * 0.4f));
			yield return new WaitForSeconds (0.05f);
		}

	}



	void DrawBearHeading() {
		if (bearInstance != null) {
			LineRenderer lineRenderer = bearInstance.GetComponent<LineRenderer> ();
			lineRenderer.SetVertexCount (2);
			lineRenderer.SetPosition (0, bearInstance.transform.position);
			lineRenderer.SetPosition (1, bearInstance.transform.forward * 20 + bearInstance.transform.position);
		}
	}
}
